import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { UsersRepository } from '../../adapters/secondaries/users.repository';
import { DeleteUserCommand } from '../../commands/delete-user.command';
import { User } from '../entities/user';

@CommandHandler(DeleteUserCommand)
export class DeleteUserUseCase {
  constructor(
    private readonly _repository: UsersRepository,
    private readonly _messager: Messager,
  ) {}

  async execute(command: DeleteUserCommand): Promise<User> {
    try {
      const user = await this._repository.delete(command.uuid);
      this._messager.publish(
        'user.delete',
        JSON.stringify({ uuid: user.uuid }),
      );
      this._messager.publish(
        'logging.user.delete.info',
        JSON.stringify({ uuid: user.uuid }),
      );
      return user;
    } catch (error) {
      this._messager.publish(
        'logging.user.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
